@@ -403,9 +403,9 @@ void pidgin_dialogs_about()
 	GtkWidget *logo;
 	GtkWidget *frame;
 	GtkWidget *text;
-#if !HILDON_CHECK_VERSION(2,1,0)
+#ifdef USE_CLOSE_AND_CANCEL_BUTTONS
 	GtkWidget *button;
-#endif /* HILDON < 2.1.0 */
+#endif /* USE_CLOSE_AND_CANCEL_BUTTONS */
 	GtkTextIter iter;
 	GString *str;
 	AtkObject *obj;
@@ -699,20 +699,20 @@ if (purple_plugins_find_with_id("core-tcl") != NULL) {
 	gtk_text_buffer_get_start_iter(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)), &iter);
 	gtk_text_buffer_place_cursor(gtk_text_view_get_buffer(GTK_TEXT_VIEW(text)), &iter);
 
-#if !HILDON_CHECK_VERSION(2,1,0)
+#ifdef USE_CLOSE_AND_CANCEL_BUTTONS
 	/* Close Button */
 	button = pidgin_dialog_add_button(GTK_DIALOG(about), GTK_STOCK_CLOSE,
 	                G_CALLBACK(destroy_about), about);
-#endif /* HILDON < 2.1.0 */
+#endif /* USE_CLOSE_AND_CANCEL_BUTTONS */
 
 	g_signal_connect(G_OBJECT(about), "destroy",
 					 G_CALLBACK(destroy_about), G_OBJECT(about));
 
-#if !HILDON_CHECK_VERSION(2,1,0)
+#ifdef USE_CLOSE_AND_CANCEL_BUTTONS
 	/* this makes the sizes not work? */
 	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
 	gtk_widget_grab_default(button);
-#endif /* HILDON < 2.1.0 */
+#endif /* USE_CLOSE_AND_CANCEL_BUTTONS */
 
 	/* Let's give'em something to talk about -- woah woah woah */
 	buddylist = pidgin_blist_get_default_gtk_blist();
