@@ -39,7 +39,6 @@
 #include <gdk/gdkkeysyms.h>
 
 static GtkWidget * create_account_field(PurpleRequestField *field);
-static char *next_ui_hint = NULL;
 
 typedef struct
 {
@@ -81,55 +80,6 @@ typedef struct
 
 } PidginRequestData;
 
-static GtkWindow *
-find_toplevel(GList *ll_toplevels, const char *role)
-{
-	const char *window_role = NULL;
-	GList *ll_itr = NULL;
-
-	for (ll_itr = ll_toplevels ; ll_itr ; ll_itr = ll_itr->next) {
-		if ((window_role = gtk_window_get_role(GTK_WINDOW(ll_itr->data))) != NULL) {
-			if (!strcmp(window_role, role))
-				return GTK_WINDOW(ll_itr->data);
-		}
-	}
-
-	return NULL;
-}
-
-static GtkWindow *
-get_request_parent(const char *ui_hint, PidginConversation *convo)
-{
-	GtkWindow *toplevel = NULL;
-	PidginBuddyList *blist = NULL;
-
-	if (convo)
-		return GTK_WINDOW(convo->win->window);
-
-	if (strcmp(ui_hint, PURPLE_REQUEST_UI_HINT_BLIST)) {
-		GList *ll_toplevels = NULL;
-
-		ll_toplevels = gtk_window_list_toplevels();
-
-		if (!(toplevel = find_toplevel(ll_toplevels, ui_hint))) {
-			if (!strcmp(ui_hint, PURPLE_REQUEST_UI_HINT_REGISTER))
-				toplevel = find_toplevel(ll_toplevels, "account");
-			else
-			if (!strcmp(ui_hint, PURPLE_REQUEST_UI_HINT_XFER))
-				toplevel = find_toplevel(ll_toplevels, "file transfer");
-		}
-		
-		g_list_free(ll_toplevels);
-	}
-
-	/* Takes care of "pidgin-statusbox" as well */
-	if (!toplevel)
-		if ((blist = pidgin_blist_get_default_gtk_blist()) != NULL)
-			return GTK_WINDOW(blist->window);
-
-	return toplevel;
-}
-
 static void
 generic_response_start(PidginRequestData *data)
 {
@@ -301,11 +251,12 @@ multifield_cancel_cb(GtkWidget *button, PidginRequestData *data)
 	purple_request_close(PURPLE_REQUEST_FIELDS, data);
 }
 
-static void
+static gboolean
 destroy_multifield_cb(GtkWidget *dialog, GdkEvent *event,
 					  PidginRequestData *data)
 {
 	multifield_cancel_cb(NULL, data);
+	return FALSE;
 }
 
 
@@ -363,7 +314,7 @@ pidgin_request_input(const char *title, const char *primary,
 
 	/* Create the dialog. */
 	dialog = gtk_dialog_new_with_buttons(title ? title : PIDGIN_ALERT_TITLE,
-					     get_request_parent(next_ui_hint, conv ? PIDGIN_CONVERSATION(conv) : NULL), 0,
+					     NULL, 0,
 					     text_to_stock(cancel_text), 1,
 					     text_to_stock(ok_text),     0,
 					     NULL);
@@ -433,19 +384,11 @@ pidgin_request_input(const char *title, const char *primary,
 			gtk_imhtml_append_text(GTK_IMHTML(entry), default_value, GTK_IMHTML_NO_SCROLL);
 		gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 		gtk_widget_show(frame);
+
+		gtk_imhtml_set_return_inserts_newline(GTK_IMHTML(entry));
 	}
 	else {
 		if (multiline) {
-			GtkWidget *sw;
-
-			sw = gtk_scrolled_window_new(NULL, NULL);
-			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-										   GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-			gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-												GTK_SHADOW_IN);
-
-			gtk_widget_set_size_request(sw, 320, 130);
-
 			/* GtkTextView */
 			entry = gtk_text_view_new();
 			gtk_text_view_set_editable(GTK_TEXT_VIEW(entry), TRUE);
@@ -459,12 +402,12 @@ pidgin_request_input(const char *title, const char *primary,
 
 			gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(entry), GTK_WRAP_WORD_CHAR);
 
-			gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
-
 			if (purple_prefs_get_bool(PIDGIN_PREFS_ROOT "/conversations/spellcheck"))
 				pidgin_setup_gtkspell(GTK_TEXT_VIEW(entry));
 
-			gtk_container_add(GTK_CONTAINER(sw), entry);
+			gtk_box_pack_start(GTK_BOX(vbox), 
+				pidgin_make_scrollable(entry, GTK_POLICY_NEVER, GTK_POLICY_ALWAYS, GTK_SHADOW_IN, 320, 130),
+				TRUE, TRUE, 0);
 		}
 		else {
 			entry = gtk_entry_new();
@@ -479,8 +422,10 @@ pidgin_request_input(const char *title, const char *primary,
 			if (masked)
 			{
 				gtk_entry_set_visibility(GTK_ENTRY(entry), FALSE);
+#if !GTK_CHECK_VERSION(2,16,0)
 				if (gtk_entry_get_invisible_char(GTK_ENTRY(entry)) == '*')
 					gtk_entry_set_invisible_char(GTK_ENTRY(entry), PIDGIN_INVISIBLE_CHAR);
+#endif /* Less than GTK+ 2.16 */
 #ifdef USE_HILDON
 				hildon_gtk_entry_set_input_mode(GTK_ENTRY(entry),
 					hildon_gtk_entry_get_input_mode(GTK_ENTRY(entry)) & ~(HILDON_GTK_INPUT_MODE_AUTOCAP));
@@ -493,6 +438,8 @@ pidgin_request_input(const char *title, const char *primary,
 	pidgin_set_accessible_label (entry, label);
 	data->u.input.entry = entry;
 
+	pidgin_auto_parent_window(dialog);
+
 	/* Show everything. */
 	gtk_widget_show(dialog);
 
@@ -529,8 +476,6 @@ pidgin_request_choice(const char *title, const char *primary,
 
 	/* Create the dialog. */
 	data->dialog = dialog = gtk_dialog_new();
-	gtk_window_set_transient_for(GTK_WINDOW(dialog),
-		get_request_parent(next_ui_hint, conv ? PIDGIN_CONVERSATION(conv) : NULL));
 
 	if (title != NULL)
 		gtk_window_set_title(GTK_WINDOW(dialog), title);
@@ -602,6 +547,8 @@ pidgin_request_choice(const char *title, const char *primary,
 	g_object_set_data(G_OBJECT(dialog), "radio", radio);
 
 	/* Show everything. */
+	pidgin_auto_parent_window(dialog);
+
 	gtk_widget_show_all(dialog);
 
 	return data;
@@ -641,8 +588,6 @@ pidgin_request_action(const char *title, const char *primary,
 
 	/* Create the dialog. */
 	data->dialog = dialog = gtk_dialog_new();
-	gtk_window_set_transient_for(GTK_WINDOW(dialog),
-		get_request_parent(next_ui_hint, conv ? PIDGIN_CONVERSATION(conv) : NULL));
 
 #if GTK_CHECK_VERSION(2,10,0)
 	gtk_window_set_deletable(GTK_WINDOW(data->dialog), FALSE);
@@ -716,9 +661,15 @@ pidgin_request_action(const char *title, const char *primary,
 		gtk_widget_grab_focus(img);
 		gtk_widget_grab_default(img);
 	} else
-		gtk_dialog_set_default_response(GTK_DIALOG(dialog), default_action);
+		/*
+		 * Need to invert the default_action number because the
+		 * buttons are added to the dialog in reverse order.
+		 */
+		gtk_dialog_set_default_response(GTK_DIALOG(dialog), action_count - 1 - default_action);
 
 	/* Show everything. */
+	pidgin_auto_parent_window(dialog);
+
 	gtk_widget_show_all(dialog);
 
 	return data;
@@ -727,15 +678,17 @@ pidgin_request_action(const char *title, const char *primary,
 static void
 req_entry_field_changed_cb(GtkWidget *entry, PurpleRequestField *field)
 {
+	PurpleRequestFieldGroup *group;
 	PidginRequestData *req_data;
 	const char *text = gtk_entry_get_text(GTK_ENTRY(entry));
 
 	purple_request_field_string_set_value(field, (*text == '\0' ? NULL : text));
 
-	req_data = (PidginRequestData *)field->group->fields_list->ui_data;
+	group = purple_request_field_get_group(field);
+	req_data = (PidginRequestData *)group->fields_list->ui_data;
 
 	gtk_widget_set_sensitive(req_data->ok_button,
-		purple_request_fields_all_required_filled(field->group->fields_list));
+		purple_request_fields_all_required_filled(group->fields_list));
 }
 
 static void
@@ -756,20 +709,27 @@ setup_entry_field(GtkWidget *entry, PurpleRequestField *field)
 		if (purple_str_has_prefix(type_hint, "screenname"))
 		{
 			GtkWidget *optmenu = NULL;
-			GList *fields = field->group->fields;
+			PurpleRequestFieldGroup *group = purple_request_field_get_group(field);
+			GList *fields = group->fields;
+
+			/* Ensure the account option menu is created (if the widget hasn't
+			 * been initialized already) for username auto-completion. */
 			while (fields)
 			{
 				PurpleRequestField *fld = fields->data;
 				fields = fields->next;
 
-				if (purple_request_field_get_type(fld) == PURPLE_REQUEST_FIELD_ACCOUNT)
+				if (purple_request_field_get_type(fld) == PURPLE_REQUEST_FIELD_ACCOUNT &&
+						purple_request_field_is_visible(fld))
 				{
 					const char *type_hint = purple_request_field_get_type_hint(fld);
 					if (type_hint != NULL && strcmp(type_hint, "account") == 0)
 					{
-						if (fld->ui_data == NULL)
-							fld->ui_data = create_account_field(fld);
-						optmenu = GTK_WIDGET(fld->ui_data);
+						optmenu = GTK_WIDGET(purple_request_field_get_ui_data(fld));
+						if (optmenu == NULL) {
+							optmenu = GTK_WIDGET(create_account_field(fld));
+							purple_request_field_set_ui_data(fld, optmenu);
+						}
 						break;
 					}
 				}
@@ -791,12 +751,6 @@ create_string_field(PurpleRequestField *field)
 	{
 		GtkWidget *textview;
 
-		widget = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(widget),
-											GTK_SHADOW_IN);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(widget),
-									   GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
-
 		textview = gtk_text_view_new();
 		gtk_text_view_set_editable(GTK_TEXT_VIEW(textview),
 								   TRUE);
@@ -806,11 +760,8 @@ create_string_field(PurpleRequestField *field)
 		if (purple_prefs_get_bool(PIDGIN_PREFS_ROOT "/conversations/spellcheck"))
 			pidgin_setup_gtkspell(GTK_TEXT_VIEW(textview));
 
-		gtk_container_add(GTK_CONTAINER(widget), textview);
 		gtk_widget_show(textview);
 
-		gtk_widget_set_size_request(widget, -1, 75);
-
 		if (value != NULL)
 		{
 			GtkTextBuffer *buffer;
@@ -825,6 +776,7 @@ create_string_field(PurpleRequestField *field)
 
 		g_signal_connect(G_OBJECT(textview), "focus-out-event",
 						 G_CALLBACK(field_string_focus_out_cb), field);
+		widget = pidgin_make_scrollable(textview, GTK_POLICY_NEVER, GTK_POLICY_ALWAYS, GTK_SHADOW_IN, -1, 75);
 	}
 	else
 	{
@@ -838,8 +790,10 @@ create_string_field(PurpleRequestField *field)
 		if (purple_request_field_string_is_masked(field))
 		{
 			gtk_entry_set_visibility(GTK_ENTRY(widget), FALSE);
+#if !GTK_CHECK_VERSION(2,16,0)
 			if (gtk_entry_get_invisible_char(GTK_ENTRY(widget)) == '*')
 				gtk_entry_set_invisible_char(GTK_ENTRY(widget),	PIDGIN_INVISIBLE_CHAR);
+#endif /* Less than GTK+ 2.16 */
 #ifdef USE_HILDON
 			hildon_gtk_entry_set_input_mode(GTK_ENTRY(widget),
 				hildon_gtk_entry_get_input_mode(GTK_ENTRY(widget)) & ~(HILDON_GTK_INPUT_MODE_AUTOCAP));
@@ -883,13 +837,25 @@ create_int_field(PurpleRequestField *field)
 	return widget;
 }
 
+#define BOOL_FIELD_BUTTON_USE_LABEL
+#ifdef USE_HILDON
+#	if HILDON_CHECK_VERSION(2,1,0)
+#		undef BOOL_FIELD_BUTTON_USE_LABEL
+#	endif /* HILDON >= 2.1.0 */
+#endif /* USE_HILDON */
+
 static GtkWidget *
 create_bool_field(PurpleRequestField *field)
 {
 	GtkWidget *widget;
 
+#ifdef BOOL_FIELD_BUTTON_USE_LABEL
 	widget = gtk_check_button_new_with_label(
 		purple_request_field_get_label(field));
+#else /* !BOOL_FIELD_BUTTON_USE_LABEL */
+	widget = gtk_check_button_new_with_mnemonic(
+		purple_request_field_get_label(field));
+#endif /* BOOL_FIELD_BUTTON_USE_LABEL */
 
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget),
 		purple_request_field_bool_get_default_value(field));
@@ -904,12 +870,11 @@ static GtkWidget *
 create_choice_field(PurpleRequestField *field)
 {
 	GtkWidget *widget;
-	GList *labels;
+	GList *labels = purple_request_field_choice_get_labels(field);
+	int num_labels = g_list_length(labels);
 	GList *l;
 
-	labels = purple_request_field_choice_get_labels(field);
-
-	if (g_list_length(labels) > 5)
+	if (num_labels > 5)
 	{
 		widget = gtk_combo_box_new_text();
 
@@ -929,7 +894,7 @@ create_choice_field(PurpleRequestField *field)
 		GtkWidget *radio;
 		gint i;
 
-		if (g_list_length(labels) == 2)
+		if (num_labels == 2)
 			box = gtk_hbox_new(FALSE, PIDGIN_HIG_BOX_SPACE);
 		else
 			box = gtk_vbox_new(FALSE, 0);
@@ -1025,7 +990,6 @@ list_field_select_changed_cb(GtkTreeSelection *sel, PurpleRequestField *field)
 static GtkWidget *
 create_list_field(PurpleRequestField *field)
 {
-	GtkWidget *sw;
 	GtkWidget *treeview;
 	GtkListStore *store;
 	GtkCellRenderer *renderer;
@@ -1034,20 +998,12 @@ create_list_field(PurpleRequestField *field)
 	GtkTreeIter iter;
 	GList *l;
 
-	/* Create the scrolled window */
-	sw = gtk_scrolled_window_new(NULL, NULL);
-	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-				       GTK_POLICY_AUTOMATIC,
-				       GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-										GTK_SHADOW_IN);
-	gtk_widget_show(sw);
-
 	/* Create the list store */
 	store = gtk_list_store_new(2, G_TYPE_POINTER, G_TYPE_STRING);
 
 	/* Create the tree view */
 	treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+	g_object_unref(G_OBJECT(store));
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(treeview), FALSE);
 
 	sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
@@ -1088,10 +1044,9 @@ create_list_field(PurpleRequestField *field)
 	g_signal_connect(G_OBJECT(sel), "changed",
 					 G_CALLBACK(list_field_select_changed_cb), field);
 
-	gtk_container_add(GTK_CONTAINER(sw), treeview);
 	gtk_widget_show(treeview);
 
-	return sw;
+	return pidgin_make_scrollable(treeview, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC, GTK_SHADOW_IN, -1, -1);
 }
 
 static void *
@@ -1107,13 +1062,11 @@ pidgin_request_fields(const char *title, const char *primary,
 	GtkWidget *vbox;
 	GtkWidget *vbox2;
 	GtkWidget *hbox;
-	GtkWidget *bbox;
 	GtkWidget *frame;
 	GtkWidget *label;
 	GtkWidget *table;
 	GtkWidget *button;
 	GtkWidget *img;
-	GtkWidget *sw;
 	GtkSizeGroup *sg;
 	GList *gl, *fl;
 	PurpleRequestFieldGroup *group;
@@ -1135,20 +1088,19 @@ pidgin_request_fields(const char *title, const char *primary,
 	data->cbs[0] = ok_cb;
 	data->cbs[1] = cancel_cb;
 
+
 #ifdef _WIN32
-	data->dialog = win = pidgin_create_window(PIDGIN_ALERT_TITLE, PIDGIN_HIG_BORDER, "multifield", TRUE) ;
+	data->dialog = win = pidgin_create_dialog(PIDGIN_ALERT_TITLE, PIDGIN_HIG_BORDER, "multifield", TRUE) ;
 #else /* !_WIN32 */
-	data->dialog = win = pidgin_create_window(title, PIDGIN_HIG_BORDER, "multifield", TRUE) ;
+	data->dialog = win = pidgin_create_dialog(title, PIDGIN_HIG_BORDER, "multifield", TRUE) ;
 #endif /* _WIN32 */
-	gtk_window_set_transient_for(GTK_WINDOW(win),
-		get_request_parent(next_ui_hint, conv ? PIDGIN_CONVERSATION(conv) : NULL));
 
 	g_signal_connect(G_OBJECT(win), "delete_event",
 					 G_CALLBACK(destroy_multifield_cb), data);
 
 	/* Setup the main horizontal box */
 	hbox = gtk_hbox_new(FALSE, PIDGIN_HIG_BORDER);
-	gtk_container_add(GTK_CONTAINER(win), hbox);
+	gtk_container_add(GTK_CONTAINER(pidgin_dialog_get_vbox(GTK_DIALOG(win))), hbox);
 	gtk_widget_show(hbox);
 
 	/* Dialog icon. */
@@ -1158,6 +1110,16 @@ pidgin_request_fields(const char *title, const char *primary,
 	gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
 	gtk_widget_show(img);
 
+	/* Cancel button */
+	button = pidgin_dialog_add_button(GTK_DIALOG(win), text_to_stock(cancel_text), G_CALLBACK(multifield_cancel_cb), data);
+	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+
+	/* OK button */
+	button = pidgin_dialog_add_button(GTK_DIALOG(win), text_to_stock(ok_text), G_CALLBACK(multifield_ok_cb), data);
+	data->ok_button = button;
+	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_window_set_default(GTK_WINDOW(win), button);
+
 	/* Setup the vbox */
 	vbox = gtk_vbox_new(FALSE, PIDGIN_HIG_BORDER);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 0);
@@ -1187,18 +1149,10 @@ pidgin_request_fields(const char *title, const char *primary,
 	if(total_fields > 9) {
 		GtkWidget *hbox_for_spacing, *vbox_for_spacing;
 
-		sw = gtk_scrolled_window_new(NULL, NULL);
-		gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
-				GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-		gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(sw),
-				GTK_SHADOW_NONE);
-		gtk_widget_set_size_request(sw, -1, 200);
-		gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
-		gtk_widget_show(sw);
-
 		hbox_for_spacing = gtk_hbox_new(FALSE, PIDGIN_HIG_BORDER);
-		gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw),
-				hbox_for_spacing);
+		gtk_box_pack_start(GTK_BOX(vbox), 
+			pidgin_make_scrollable(hbox_for_spacing, GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC, GTK_SHADOW_NONE, -1, 200), 
+			TRUE, TRUE, 0);
 		gtk_widget_show(hbox_for_spacing);
 
 		vbox_for_spacing = gtk_vbox_new(FALSE, PIDGIN_HIG_BORDER);
@@ -1310,6 +1264,7 @@ pidgin_request_fields(const char *title, const char *primary,
 				size_t col_offset = col_num * 2;
 				PurpleRequestFieldType type;
 				GtkWidget *widget = NULL;
+				const char *field_label;
 
 				label = NULL;
 				field = fl->data;
@@ -1320,17 +1275,17 @@ pidgin_request_fields(const char *title, const char *primary,
 				}
 
 				type = purple_request_field_get_type(field);
+				field_label = purple_request_field_get_label(field);
 
-				if (type != PURPLE_REQUEST_FIELD_BOOLEAN &&
-				    purple_request_field_get_label(field))
+				if (type != PURPLE_REQUEST_FIELD_BOOLEAN && field_label)
 				{
-					char *text;
+					char *text = NULL;
 
-					text = g_strdup_printf("%s:",
-						purple_request_field_get_label(field));
+					if (field_label[strlen(field_label) - 1] != ':')
+						text = g_strdup_printf("%s:", field_label);
 
 					label = gtk_label_new(NULL);
-					gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), text);
+					gtk_label_set_markup_with_mnemonic(GTK_LABEL(label), text ? text : field_label);
 					g_free(text);
 
 					gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
@@ -1362,24 +1317,26 @@ pidgin_request_fields(const char *title, const char *primary,
 					gtk_widget_show(label);
 				}
 
-				if (field->ui_data != NULL)
-					widget = GTK_WIDGET(field->ui_data);
-				else if (type == PURPLE_REQUEST_FIELD_STRING)
-					widget = create_string_field(field);
-				else if (type == PURPLE_REQUEST_FIELD_INTEGER)
-					widget = create_int_field(field);
-				else if (type == PURPLE_REQUEST_FIELD_BOOLEAN)
-					widget = create_bool_field(field);
-				else if (type == PURPLE_REQUEST_FIELD_CHOICE)
-					widget = create_choice_field(field);
-				else if (type == PURPLE_REQUEST_FIELD_LIST)
-					widget = create_list_field(field);
-				else if (type == PURPLE_REQUEST_FIELD_IMAGE)
-					widget = create_image_field(field);
-				else if (type == PURPLE_REQUEST_FIELD_ACCOUNT)
-					widget = create_account_field(field);
-				else
-					continue;
+				widget = GTK_WIDGET(purple_request_field_get_ui_data(field));
+				if (widget == NULL)
+				{
+					if (type == PURPLE_REQUEST_FIELD_STRING)
+						widget = create_string_field(field);
+					else if (type == PURPLE_REQUEST_FIELD_INTEGER)
+						widget = create_int_field(field);
+					else if (type == PURPLE_REQUEST_FIELD_BOOLEAN)
+						widget = create_bool_field(field);
+					else if (type == PURPLE_REQUEST_FIELD_CHOICE)
+						widget = create_choice_field(field);
+					else if (type == PURPLE_REQUEST_FIELD_LIST)
+						widget = create_list_field(field);
+					else if (type == PURPLE_REQUEST_FIELD_IMAGE)
+						widget = create_image_field(field);
+					else if (type == PURPLE_REQUEST_FIELD_ACCOUNT)
+						widget = create_account_field(field);
+					else
+						continue;
+				}
 
 				if (label)
 					gtk_label_set_mnemonic_widget(GTK_LABEL(label), widget);
@@ -1424,45 +1381,17 @@ pidgin_request_fields(const char *title, const char *primary,
 
 				gtk_widget_show(widget);
 
-				field->ui_data = widget;
+				purple_request_field_set_ui_data(field, widget);
 			}
 		}
 	}
 
 	g_object_unref(sg);
 
-	/* Button box. */
-	bbox = gtk_hbutton_box_new();
-	gtk_box_set_spacing(GTK_BOX(bbox), PIDGIN_HIG_BOX_SPACE);
-	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
-	gtk_box_pack_end(GTK_BOX(vbox), bbox, FALSE, TRUE, 0);
-	gtk_widget_show(bbox);
-
-	/* Cancel button */
-	button = gtk_button_new_from_stock(text_to_stock(cancel_text));
-	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-	gtk_widget_show(button);
-
-	g_signal_connect(G_OBJECT(button), "clicked",
-					 G_CALLBACK(multifield_cancel_cb), data);
-
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-
-	/* OK button */
-	button = gtk_button_new_from_stock(text_to_stock(ok_text));
-	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-	gtk_widget_show(button);
-
-	data->ok_button = button;
-
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
-	gtk_window_set_default(GTK_WINDOW(win), button);
-
-	g_signal_connect(G_OBJECT(button), "clicked",
-					 G_CALLBACK(multifield_ok_cb), data);
-
 	if (!purple_request_fields_all_required_filled(fields))
-		gtk_widget_set_sensitive(button, FALSE);
+		gtk_widget_set_sensitive(data->ok_button, FALSE);
+
+	pidgin_auto_parent_window(win);
 
 	gtk_widget_show(win);
 
@@ -1551,13 +1480,15 @@ file_ok_check_if_exists_cb(GtkWidget *button, PidginRequestData *data)
 	}
 
 #endif /* FILECHOOSER */
-
+#if 0 /* mismatched curly braces */
+	}
+#endif
 	if ((data->u.file.savedialog == TRUE) &&
 		(g_file_test(data->u.file.name, G_FILE_TEST_EXISTS))) {
-		purple_request_action_with_hint(data, NULL, _("That file already exists"),
+		purple_request_action(data, NULL, _("That file already exists"),
 							_("Would you like to overwrite it?"), 0,
 							NULL, NULL, NULL,
-							"pidgin-request-file", data, 2,
+							data, 2,
 							_("Overwrite"), G_CALLBACK(file_yes_no_cb),
 							_("Choose New Name"), G_CALLBACK(file_yes_no_cb));
 	} else
@@ -1565,7 +1496,7 @@ file_ok_check_if_exists_cb(GtkWidget *button, PidginRequestData *data)
 }
 
 #if !GTK_CHECK_VERSION(2,4,0) /* FILECHOOSER */
-static void
+static gboolean
 file_cancel_cb(PidginRequestData *data)
 {
 	generic_response_start(data);
@@ -1574,73 +1505,10 @@ file_cancel_cb(PidginRequestData *data)
 		((PurpleRequestFileCb)data->cbs[0])(data->user_data, NULL);
 
 	purple_request_close(data->type, data);
+	return FALSE;
 }
 #endif /* FILECHOOSER */
 
-#ifdef USE_HILDON
-/*
- * On Maemo, the sole user on the system, aptly named "user", has the directory ~/MyDocs/.{images,documents,videos,etc.}.
- * This directory tree is treated specially by the file system widgets. They hide all of /, and display only these directories.
- * For pidgin, this is unacceptable, because the user sometimes has to browse /usr/share/pidgin/sounds, for example. However,
- * we should add these directories here to better integrate with the rest of the Maemo setup.
- *
- * Obviously, add_MyDocs does nothing if there's no ~/MyDocs.
- */
-static void
-add_MyDocs(GtkWidget *file_chooser)
-{
-	char *my_docs_dir = NULL ;
-	GDir *dir = NULL;
-	char *entry = NULL;
-
-	/* Yes, this stuff is thoroughly hard-coded, but it only happens on Maemo, and is a NOOP otherwise */
-	if ((my_docs_dir = g_strdup_printf ("%s%sMyDocs", getenv ("HOME"), G_DIR_SEPARATOR_S)) != NULL) {
-		if ((dir = g_dir_open(my_docs_dir, 0, NULL)) != NULL) {
-			while ((entry = (char *)g_dir_read_name(dir)) != NULL) {
-				entry = g_strdup_printf("/home/user/MyDocs%s%s", G_DIR_SEPARATOR_S, entry);
-				gtk_file_chooser_add_shortcut_folder(GTK_FILE_CHOOSER(file_chooser), entry, NULL);
-				g_free(entry);
-			}
-			g_dir_close(dir);
-		}
-	g_free(my_docs_dir);
-	}
-}
-
-static void show_hidden_notify_active (GtkWidget *widget, GParamSpec *pspec, GtkWidget *file_chooser)
-{
-	gboolean active;
-
-	g_object_get(G_OBJECT(widget), "active", &active, NULL);
-	/* This doesn't work for some reason. Hazaa for consistency ! */
-	/* g_object_set(G_OBJECT(widget), "show-hidden", active, NULL); */
-	gtk_file_chooser_set_show_hidden(GTK_FILE_CHOOSER(file_chooser), active);
-}
-
-static void hildon_file_chooser_dialog_mods(GtkWidget *file_chooser)
-{
-	GtkWidget *show_hidden = NULL;
-	GtkFileChooserAction action;
-
-	g_object_get(G_OBJECT(file_chooser), "action", &action, NULL);
-
-	/* Add a "Show hidden files/folders" checkbox */
-	gtk_container_add_with_properties(GTK_CONTAINER(GTK_DIALOG (file_chooser)->vbox),
-		show_hidden = GTK_WIDGET(g_object_new(GTK_TYPE_CHECK_BUTTON, "visible", TRUE,
-			"label", GTK_FILE_CHOOSER_ACTION_OPEN == action || GTK_FILE_CHOOSER_ACTION_SAVE == action 
-					? _("Show Hidden Files")
-					: _("Show Hidden Folders"),
-			"active", gtk_file_chooser_get_show_hidden(GTK_FILE_CHOOSER (file_chooser)),
-			NULL)),
-		"expand", FALSE, "fill", TRUE, NULL);
-
-	/* Add Maemo special folders */
-	add_MyDocs(file_chooser);
-
-	g_signal_connect(G_OBJECT(show_hidden), "notify::active", (GCallback)show_hidden_notify_active, file_chooser);
-}
-#endif /* def USE_HILDON */
-
 static void *
 pidgin_request_file(const char *title, const char *filename,
 					  gboolean savedialog,
@@ -1677,7 +1545,7 @@ pidgin_request_file(const char *title, const char *filename,
 						GTK_RESPONSE_ACCEPT,
 						NULL);
 #ifdef USE_HILDON
-	hildon_file_chooser_dialog_mods(filesel);
+	_hildon_file_chooser_dialog_mods(filesel);
 #endif /* USE_HILDON */
 	gtk_dialog_set_default_response(GTK_DIALOG(filesel), GTK_RESPONSE_ACCEPT);
 
@@ -1737,9 +1605,8 @@ pidgin_request_file(const char *title, const char *filename,
 	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(filesel)->ok_button), "clicked",
 					 G_CALLBACK(file_ok_check_if_exists_cb), data);
 #endif /* FILECHOOSER */
-	gtk_window_set_role(GTK_WINDOW(filesel), "pidgin-request-file");
-	gtk_window_set_transient_for(GTK_WINDOW(filesel),
-		get_request_parent(next_ui_hint, conv ? PIDGIN_CONVERSATION(conv) : NULL));
+
+	pidgin_auto_parent_window(filesel);
 
 	data->dialog = filesel;
 	gtk_widget_show(filesel);
@@ -1774,7 +1641,7 @@ pidgin_request_folder(const char *title, const char *dirname,
 						GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
 						NULL);
 #ifdef USE_HILDON
-	hildon_file_chooser_dialog_mods(dirsel);
+	_hildon_file_chooser_dialog_mods(dirsel);
 #endif /* USE_HILDON */
 	gtk_dialog_set_default_response(GTK_DIALOG(dirsel), GTK_RESPONSE_ACCEPT);
 
@@ -1793,11 +1660,10 @@ pidgin_request_folder(const char *title, const char *dirname,
 	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(dirsel)->ok_button), "clicked",
 					 G_CALLBACK(file_ok_check_if_exists_cb), data);
 #endif
-	gtk_window_set_role(GTK_WINDOW(dirsel), "pidgin-request-dir");
-	gtk_window_set_transient_for(GTK_WINDOW(dirsel),
-		get_request_parent(next_ui_hint, conv ? PIDGIN_CONVERSATION(conv) : NULL));
 
 	data->dialog = dirsel;
+	pidgin_auto_parent_window(dirsel);
+
 	gtk_widget_show(dirsel);
 
 	return (void *)data;
@@ -1820,12 +1686,6 @@ pidgin_close_request(PurpleRequestType type, void *ui_handle)
 	g_free(data);
 }
 
-static void
-pidgin_request_set_ui_hint(const char *ui_hint)
-{
-	next_ui_hint = ui_hint;
-}
-
 static PurpleRequestUiOps ops =
 {
 	pidgin_request_input,
@@ -1835,7 +1695,7 @@ static PurpleRequestUiOps ops =
 	pidgin_request_file,
 	pidgin_close_request,
 	pidgin_request_folder,
-	pidgin_request_set_ui_hint,
+	NULL,
 	NULL,
 	NULL,
 	NULL
